# -*- coding: utf-8
def reformat_chromosome(chromosome):
    if 'p' in chromosome:
        new_chrom = chromosome.split('p')[0].upper()
    elif 'q' in chromosome:
        new_chrom = chromosome.split('q')[0].upper()
    else:
        new_chrom = chromosome.replace('chr', '').upper()

    if new_chrom in list(map(str, range(1, 23))) or new_chrom in ['X', 'Y', 'M', 'MT']:
        return new_chrom



import logging

from django.core.management import BaseCommand, call_command


logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Sync Genome Database'

    def add_arguments(self, parser):
        parser.add_argument(
            '--genome-build',
            choices=['hg18', 'hg19', 'hg38'],
            default='hg19',
            help='Genome build to load',
        )

    def handle(self, *args, **options):
        logger.info('Syncing {0}'.format(options['genome_build']))
        call_command('chromosome_sync', *args, **options)
        call_command('gene_sync', *args, **options)
        call_command('transcript_sync', *args, **options)



import logging

from django.core.management import BaseCommand

from genomix.utils import retrieve_compressed_data, retrieve_data

from genome import app_settings, models, utils


logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Sync Chromosomes'

    def add_arguments(self, parser):
        parser.add_argument(
            '--genome-build',
            choices=['hg18', 'hg19', 'hg38'],
            default='hg19',
            help='Genome build to load',
        )

    def handle(self, *args, **options):
        genome = options['genome_build']
        resources = app_settings.RESOURCES.get(genome, {})
        description_url = resources.get('url', "")

        # Download external resources
        logger.info('Downloading resources...')
        chromosome_data = retrieve_data(resources.get('chromosomes'))
        cytoband_data = retrieve_compressed_data(resources.get('cytoband'))

        # Create Genome
        logger.info('Creating Genome...')
        genome_obj, created = models.Genome.objects.get_or_create(
            label=genome,
            defaults={'description_url': description_url}
        )

        # Create Chromosome
        logger.info('Creating Chromosomes...')
        for line in chromosome_data:
            (label, length) = line.strip().split('\t')

            # NOTE: This is to remove weird Chromosomes
            new_label = utils.reformat_chromosome(label)
            if new_label:
                chrom_obj, created = models.Chromosome.objects.get_or_create(
                    label=new_label,
                    genome=genome_obj,
                    defaults={'length': length, }
                )

        # Create CytoBand
        logger.info('Creating CytoBands...')
        for line in cytoband_data:
            (chromosome, start, end, label, stain) = line.strip().split('\t')

            # NOTE: This is to remove weird Chromosomes
            new_chromosome = utils.reformat_chromosome(chromosome)
            if new_chromosome:
                chrom_obj = models.Chromosome.objects.get(
                    label=new_chromosome,
                    genome=genome_obj,
                )

                cytoband_obj, created = models.CytoBand.objects.get_or_create(
                    label=label,
                    chromosome=chrom_obj,
                    defaults={
                        'start': start,
                        'end': end,
                        'stain': stain,
                    }
                )



# -*- coding: utf-8 -*-
from django import forms
from django.db.models import CharField, TextField

import django_filters
from genomix.filters import DisplayChoiceFilter

from . import choices, models


class GenomeFilter(django_filters.rest_framework.FilterSet):

    class Meta:
        model = models.Genome
        fields = [
            'label',
            'description',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
        }


class GeneFilter(django_filters.rest_framework.FilterSet):

    status = DisplayChoiceFilter(choices=choices.HGNC_GENE_STATUS)

    class Meta:
        model = models.Gene
        fields = [
            'symbol',
            'name',
            'hgnc_id',
            'status',
            'active',
            'chromosome',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            },
            TextField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }


class TranscriptFilter(django_filters.rest_framework.FilterSet):

    gene = django_filters.ModelChoiceFilter(
        queryset=models.Gene.objects.all(),
        widget=forms.NumberInput,
    )
    strand = DisplayChoiceFilter(choices=choices.STRAND_TYPES)

    class Meta:
        model = models.Transcript
        fields = [
            'label',
            'active',
            'gene',
            'gene__symbol',
            'strand',
            'transcription_start',
            'transcription_end',
            'cds_start',
            'cds_end',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            }
        }


class ExonFilter(django_filters.rest_framework.FilterSet):

    transcript = django_filters.ModelChoiceFilter(
        queryset=models.Transcript.objects.all(),
        widget=forms.NumberInput,
    )

    class Meta:
        model = models.Exon
        fields = [
            'number',
            'active',
            'transcript',
            'transcript__label',
            'start',
            'end',
            'cds',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            }
        }



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.management import BaseCommand

from genomix.utils import retrieve_compressed_data

from genome import app_settings, choices, models


logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Sync Transcripts'

    def add_arguments(self, parser):
        parser.add_argument(
            '--genome-build',
            choices=['hg18', 'hg19', 'hg38'],
            default='hg19',
            help='Genome build to load',
        )

    def handle(self, *args, **options):
        genome = options['genome_build']
        resources = app_settings.RESOURCES.get(genome, {})

        # Download external resources
        logger.info('Downloading resources...')
        refgene_data = retrieve_compressed_data(resources.get('refgene'))

        # Create Transcript
        logger.info('Creating Transcripts and Exons from refGene...')
        for line in refgene_data:
            line = line.strip().split('\t')
            label = line[1]
            strand = line[3]
            transcription_start = int(line[4])
            transcription_end = int(line[5])
            cds_start = int(line[6])
            cds_end = int(line[7])
            symbol = line[12].upper()

            try:
                gene_obj = models.Gene.objects.get(symbol__iexact=symbol)
            except ObjectDoesNotExist:
                # NOTE: We skip weird genes like LOC, etc...
                continue

            transcript_obj, created = models.Transcript.objects.get_or_create(
                label=label,
                gene=gene_obj,
                defaults={
                    'strand': getattr(choices.STRAND_TYPES, strand),
                    'transcription_start': transcription_start,
                    'transcription_end': transcription_end,
                    'cds_start': cds_start,
                    'cds_end': cds_end,
                }
            )

            # Create Exon
            exon_starts = [int(x) for x in line[9].strip().split(',') if x]
            exon_ends = [int(x) for x in line[10].strip().split(',') if x]
            number_of_exons = int(line[8])
            for index, exon in enumerate(exon_starts):

                if strand == '+':  # NOTE: Positive strand exons count forward
                    number = index + 1

                elif strand == '-':  # NOTE: Negative strand exons count reverse
                    number = number_of_exons - index
                else:
                    raise ValueError('strand: {0} is not supported!'.format(strand))

                exon_obj, created = models.Exon.objects.get_or_create(
                    number=number,
                    transcript=transcript_obj,
                    start=exon_starts[index],
                    end=exon_ends[index],
                )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-09 13:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Chromosome',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=2)),
                ('length', models.PositiveIntegerField()),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Chromosome',
                'verbose_name_plural': 'Chromosomes',
            },
        ),
        migrations.CreateModel(
            name='CytoBand',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=20)),
                ('start', models.PositiveIntegerField()),
                ('end', models.PositiveIntegerField()),
                ('stain', models.CharField(max_length=10)),
                ('active', models.BooleanField(default=True)),
                ('chromosome', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cytobands', to='genome.Chromosome')),
            ],
            options={
                'verbose_name': 'Cytogenetic Band',
                'verbose_name_plural': 'Cytogenetic Bands',
            },
        ),
        migrations.CreateModel(
            name='Exon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('number', models.PositiveSmallIntegerField()),
                ('active', models.BooleanField(default=True)),
                ('start', models.PositiveIntegerField()),
                ('end', models.PositiveIntegerField()),
                ('cds', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Exon',
                'verbose_name_plural': 'Exons',
            },
        ),
        migrations.CreateModel(
            name='Gene',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('symbol', models.CharField(db_index=True, max_length=255, unique=True)),
                ('name', models.TextField(blank=True)),
                ('hgnc_id', models.PositiveIntegerField(blank=True, null=True)),
                ('status', models.PositiveSmallIntegerField(choices=[(1, 'approved'), (2, 'entry_withdrawn'), (3, 'symbol_withdrawn'), (4, 'ucsc_gene')])),
                ('active', models.BooleanField(default=True)),
                ('previous_name', models.TextField(blank=True)),
                ('locus_type', models.CharField(blank=True, max_length=100)),
                ('locus_group', models.CharField(blank=True, max_length=100)),
                ('ensembl', models.TextField(blank=True)),
                ('refseq', models.TextField(blank=True)),
                ('not_curated_ensembl', models.TextField(blank=True, verbose_name='Ensembl ID (supplied by Ensembl)')),
                ('not_curated_refseq', models.TextField(blank=True, verbose_name='RefSeq (supplied by NCBI)')),
                ('not_curated_ucsc', models.TextField(blank=True, verbose_name='UCSC ID (supplied by UCSC)')),
                ('not_curated_omim', models.TextField(blank=True, verbose_name='OMIM ID (supplied by OMIM)')),
                ('not_curated_uniprot', models.TextField(blank=True, verbose_name='UniProt ID (supplied by UniProt)')),
                ('not_curated_mouse_genome_database', models.TextField(blank=True, verbose_name='Mouse Genome Database ID (supplied by MGI)')),
                ('not_curated_rat_genome_database', models.TextField(blank=True, verbose_name='Rat Genome Database ID (supplied by RGD)')),
                ('chromosome', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='genes', to='genome.Chromosome')),
            ],
            options={
                'verbose_name': 'Gene',
                'verbose_name_plural': 'Genes',
            },
        ),
        migrations.CreateModel(
            name='GeneSynonym',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(db_index=True, max_length=255, unique=True)),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Gene Synonym',
                'verbose_name_plural': 'Gene Synonyms',
            },
        ),
        migrations.CreateModel(
            name='Genome',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=50, unique=True)),
                ('description', models.CharField(blank=True, max_length=255)),
                ('slug', models.SlugField(unique=True)),
                ('active', models.BooleanField(default=True)),
                ('description_url', models.URLField(blank=True)),
            ],
            options={
                'verbose_name': 'Genome Build',
                'verbose_name_plural': 'Genome Builds',
            },
        ),
        migrations.CreateModel(
            name='Transcript',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(db_index=True, max_length=255, unique=True)),
                ('active', models.BooleanField(default=True)),
                ('strand', models.PositiveSmallIntegerField(choices=[(1, '+'), (2, '-')])),
                ('transcription_start', models.PositiveIntegerField()),
                ('transcription_end', models.PositiveIntegerField()),
                ('cds_start', models.PositiveIntegerField()),
                ('cds_end', models.PositiveIntegerField()),
                ('gene', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transcripts', to='genome.Gene')),
            ],
            options={
                'verbose_name': 'Transcript',
                'verbose_name_plural': 'Transcripts',
            },
        ),
        migrations.AddField(
            model_name='gene',
            name='synonyms',
            field=models.ManyToManyField(blank=True, related_name='genes', to='genome.GeneSynonym'),
        ),
        migrations.AddField(
            model_name='exon',
            name='transcript',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='exons', to='genome.Transcript'),
        ),
        migrations.AddField(
            model_name='chromosome',
            name='genome',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='chromosomes', to='genome.Genome'),
        ),
        migrations.AlterUniqueTogether(
            name='chromosome',
            unique_together=set([('genome', 'label')]),
        ),
    ]
